
-----------------------------------------------------------------
--       StatBlockCore, a core for informational blocks        --
-----------------------------------------------------------------

local SBC = _G.CreateFrame("Frame", "StatBlockCore")
local L = _G.LibStub("AceLocale-3.0"):GetLocale("StatBlockCore", true)

local ldb = _G.LibStub("LibDataBroker-1.1")
local media = _G.LibStub("LibSharedMedia-3.0")
local pairs = _G.pairs
local SBC_block = {}
local SBC_display = {}
local SBC_icon = {}
local db

------------------------------
--         Config           --
------------------------------

local options
local function getOptions()
	if not options then
		options = {
			name = "StatBlockCore",
			childGroups = "tab", type = "group",
			args = {
				main = {
					name = _G["ALL"],
					order = 1, type = "group",
					args = {
						header = {
							name = L["A core for your informational blocks"],
							order = 1, type = "header",
						},
						globalvaluecolor = {
							name = L["Global Value Color"],
							order = 2, type = "color", width = "full",
							get = function() return db.globalColor.r, db.globalColor.g, db.globalColor.b end,
							set = function(_, r, g, b)
								db.globalColor.r = r db.globalColor.g = g db.globalColor.b = b
								for k,v in pairs(SBC_display) do
									db.plugins[k].color.r = r db.plugins[k].color.g = g db.plugins[k].color.b = b
									v:SetTextColor(r,g,b)
								end
							end,
						},
						globallabelcolor = {
							name = L["Global Label Color"],
							order = 3, type = "color", width = "full",
							get = function() return db.globalLabelColor.r, db.globalLabelColor.g, db.globalLabelColor.b end,
							set = function(_, r, g, b)
								db.globalLabelColor.r = r db.globalLabelColor.g = g db.globalLabelColor.b = b
								local c = ("%02x%02x%02x"):format(r*255, g*255, b*255)
								for k in pairs(SBC_block) do
									local obj = ldb:GetDataObjectByName(k)
									if obj.suffix then
										db.plugins[k].labelc = c
										db.plugins[k].labelColor.r = r db.plugins[k].labelColor.g = g db.plugins[k].labelColor.b = b
										SBC:ValueUpdate(nil, k, nil, obj.value, obj)
									end
								end
							end,
						},
						globalfont = {
							name = L["Global Font"],
							order = 4, type = "select", width = "double",
							values = media:List("font"),
							get = function()
								for k,v in pairs(media:List("font")) do
									if db.globalFont == v then
										return k
									end
								end
							end,
							set = function(_, newfont)
								db.globalFont = media:List("font")[newfont]
								local font = media:Fetch("font", db.globalFont)
								for _,v in pairs(SBC_display) do
									v:SetFont(font, db.globalSize, db.outline)
								end
							end,
						},
						iconpos = {
							name = _G["EMBLEM_SYMBOL"], --Icon
							order = 5, type = "select", width = "double",
							values = {LEFT = L["LEFT"], RIGHT = L["RIGHT"]},
							get = function() return db.iconPos end,
							set = function(_, pos) db.iconPos = pos
								for k,v in pairs(SBC_icon) do
									local obj = ldb:GetDataObjectByName(k)
									v:ClearAllPoints()
									v:SetPoint(pos, SBC_block[k])
									if obj.icon and db.plugins[k].icon then
										SBC_display[k]:ClearAllPoints()
										SBC_display[k]:SetPoint(pos, v,
											pos == "LEFT" and "RIGHT" or "LEFT"
										)
										SBC_display[k]:SetWidth(db.plugins[k].width - 20)
									end
								end
							end,
						},
						globalalpha = {
							name = L["Global Backdrop Alpha"],
							order = 6, type = "range", width = "double",
							min = 0, max = 1, step = 0.01,
							get = function() return db.globalAlpha end,
							set = function(_, a)
								db.globalAlpha = a
								for _,v in pairs(SBC_block) do
									v:SetBackdropColor(db.globalBDC.r,db.globalBDC.g,db.globalBDC.b,a)
								end
							end,
						},
						globalsize = {
							name = L["Global Font Size"],
							order = 7, type = "range", width = "double",
							min = 1, max = 48, step = 1,
							get = function() return db.globalSize end,
							set = function(_, size)
								db.globalSize = size
								for _,v in pairs(SBC_display) do
									v:SetFont(media:Fetch("font", db.globalFont), size, db.outline)
								end
							end,
						},
						globaloutline = {
							name = L["Outline"], width = "double",
							order = 8, type = "select",
							values = {NONE = _G["NONE"], OUTLINE = L["Outline"], THICKOUTLINE = L["Thick Outline"]},
							get = function() return db.outline end,
							set = function(_, value) db.outline = value
								for _,v in pairs(SBC_display) do
									v:SetFont(media:Fetch("font", db.globalFont), db.globalSize, db.outline)
								end
							end,
						},
						globalbgcolor = {
							name = L["Global Backdrop Color"],
							order = 9, type = "color", width = "full",
							get = function() return db.globalBDC.r, db.globalBDC.g, db.globalBDC.b end,
							set = function(_, r, g, b)
								db.globalBDC.r = r db.globalBDC.g = g db.globalBDC.b = b
								for _,v in pairs(SBC_block) do
									v:SetBackdropColor(r,g,b)
								end
							end,
						},
						globalshadow = {
							name = _G["DAMAGE_SCHOOL6"],
							order = 10, type = "toggle",
							get = function() return db.shadow end,
							set = function(_, state) db.shadow = state
								if state then
									for _,v in pairs(SBC_display) do
										v:SetShadowColor(0, 0, 0, 1)
										v:SetShadowOffset(1, -1)
									end
								else
									for _,v in pairs(SBC_display) do
										v:SetShadowColor(0, 0, 0, 0)
										v:SetShadowOffset(0, 0)
									end
								end
							end,
						},
						globallock = {
							name = L["Global Lock"],
							order = 11, type = "toggle", width = "full",
							get = function() return db.globalLock end,
							set = function(_, state)
								db.globalLock = state
								for k,v in pairs(SBC_block) do
									db.plugins[k].lock = state
									local l
									if not state then l = true else l = false end
									v:SetMovable(l)
									v:EnableMouseWheel(l)
								end
							end,
						},
					},
				},
				profiles = _G.LibStub("AceDBOptions-3.0"):GetOptionsTable(SBC.db)
			},
		}
	end
	return options
end

local blockOpts = {}
local function getBlockOptions()
	local name = InterfaceOptionsFramePanelContainer.displayedFrame.name
	if not blockOpts[name] then
		local obj = ldb:GetDataObjectByName(name)
		local block = SBC_block[name]
		local pDB = db.plugins[name]
		blockOpts[name] = {
			name = name,
			order = 1, type = "group",
			args = {
				enabled = {
					name = _G["ENABLE"],
					order = 5, type = "toggle", width = "half",
					get = function() return pDB.enabled end,
					set = function(_, state) pDB.enabled = state
						if not state and block then
							block:Hide()
						elseif state and block then
							block:Show()
						else
							_G.ChatFrame1:AddMessage("|cff33ff99StatBlockCore|r: ".. L["Reload UI"])
						end
					end,
				},
				lock = block and {
					name = _G["LOCK"],
					order = 10, type = "toggle", width = "half",
					get = function() return pDB.lock end,
					set = function(_, state) pDB.lock = state
						if not state then state = true else state = false end
						block:SetMovable(state)
						block:EnableMouseWheel(state)
					end,
					disabled = function() return not pDB.enabled end,
				},
				color = block and {
					name = L["Value Color"],
					order = 20, type = "color", width = "half",
					get = function() return pDB.color.r, pDB.color.g, pDB.color.b end,
					set = function(_, r, g, b) local cdb = pDB.color 
						cdb.r = r cdb.g = g cdb.b = b SBC_display[name]:SetTextColor(r,g,b)
					end,
					disabled = function() return not pDB.enabled end,
				},
				labelcolor = block and {
					name = L["Label Color"],
					order = 30, type = "color", width = "half",
					get = function() return pDB.labelColor.r, pDB.labelColor.g, pDB.labelColor.b end,
					set = function(_, r, g, b)
						local c = ("%02x%02x%02x"):format(r*255, g*255, b*255)
						pDB.labelc = c
						pDB.labelColor.r = r pDB.labelColor.g = g pDB.labelColor.b = b
						SBC:ValueUpdate(nil, name, nil, obj.value, obj)
					end,
					disabled = function() return not obj.suffix or not pDB.enabled end,
				},
				label = block and {
					name = L["Text Display"],
					order = 40, type = "input",
					get = function() return pDB.label or obj.suffix or (obj.launcher and obj.label) or "N/A" end,
					set = function(_, label)
						pDB.label = label
						if obj.suffix then
							obj.suffix = label
							SBC:ValueUpdate(nil, name, nil, obj.value, obj)
						elseif obj.launcher then
							obj.label = label
							SBC:LauncherUpdate(name, obj.label)
						end
					end,
					disabled = function() return (not obj.suffix and not obj.launcher) or not pDB.enabled end,
				},
				align = block and {
					name = L["Font Align"],
					order = 50, type = "select", 
					values = {LEFT = L["LEFT"], CENTER = L["CENTER"], RIGHT = L["RIGHT"]},
					get = function() return pDB.align end,
					set = function(_, e) pDB.align = e SBC_display[name]:SetJustifyH(e) end,
					disabled = function() return not pDB.enabled end,
				},
				x = block and {
					name = L["X Position"],
					order = 60, type = "input",
					get = function() return tostring(pDB.posx / block:GetEffectiveScale()) end,
					set = function(_, x) x = tonumber(x)
						local s = block:GetEffectiveScale()
						pDB.posx = x * s
						block:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, pDB.posy / s)
					end,
					disabled = function() return not pDB.enabled end,
				},
				y = block and {
					name = L["Y Position"],
					order = 70, type = "input",
					get = function() return tostring(pDB.posy / block:GetEffectiveScale()) end,
					set = function(_, y) y = tonumber(y)
						local s = block:GetEffectiveScale()
						pDB.posy = y * s
						block:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", pDB.posx / s, y)
					end,
					disabled = function() return not pDB.enabled end,
				},
				scale = block and {
					name = L["Scale"],
					order = 80, type = "range", width = "full",
					min = 0.5, max = 2, step = 0.01,
					get = function() return pDB.scale end,
					set = function(_, scale) pDB.scale = scale block:SetScale(scale) end,
					disabled = function() return not pDB.enabled end,
				},
				parent = block and {
					name = L["Parenting"],
					order = 90, type = "input",
					get = function() return pDB.parent end,
					set = function(_, p)
						if _G[p] then
							pDB.parent = p
							block:SetParent(p)
						else
							_G.ChatFrame1:AddMessage("|cff33ff99StatBlockCore|r: ".. _G["ERR_TAXINOPATH"])
						end
					end,
					disabled = function() return not pDB.enabled end,
				},
				strata = block and {
					name = L["Strata"],
					order = 100, type = "select",
					get = function() return pDB.strata end,
					set = function(_, strata) pDB.strata = strata
						block:SetFrameStrata(pDB.strata)
					end,
					values = {HIGH = _G["HIGH"], MEDIUM = _G["AUCTION_TIME_LEFT2"], LOW = _G["LOW"], BACKGROUND = _G["BACKGROUND"]},
					disabled = function() return not pDB.enabled end,
				},
				hiddenalpha = block and {
					name = _G["OBJECT_ALPHA"],
					order = 110, type = "range",
					min = 0, max = 1, step = 0.01,
					get = function() return pDB.hiddenAlpha end,
					set = function(_, a) pDB.hiddenAlpha = a block:SetAlpha(a) end,
					disabled = function() return not pDB.enabled end,
				},
				shownalpha = block and {
					name = L["MouseOver Alpha"],
					order = 120, type = "range",
					min = 0, max = 1, step = 0.01,
					get = function() return pDB.shownAlpha end,
					set = function(_, a) pDB.shownAlpha = a end,
					disabled = function() return not pDB.enabled end,
				},
				icon = block and {
					name = _G["EMBLEM_SYMBOL"], --Icon
					order = 130, type = "toggle",
					get = function() return pDB.icon end,
					set = function(_, i) pDB.icon = i
						if SBC_icon[name] then
							if i then SBC_icon[name]:Show()
								SBC_display[name]:ClearAllPoints()
								SBC_display[name]:SetPoint(db.iconPos, SBC_icon[name],
									db.iconPos == "LEFT" and "RIGHT" or "LEFT"
								)
								SBC_display[name]:SetWidth(db.plugins[name].width - 20)
							else
								SBC_icon[name]:Hide()
								SBC_display[name]:ClearAllPoints()
								SBC_display[name]:SetPoint("CENTER", SBC_block[name], "CENTER")
								SBC_display[name]:SetWidth(db.plugins[name].width)
							end
						else
							SBC_icon[name] = block:CreateTexture("StatBlockCoreIcon_"..name,"BACKGROUND")
							SBC_icon[name]:SetTexture(obj.icon)
							SBC_icon[name]:SetWidth(20)
							SBC_icon[name]:SetHeight(20)
							SBC_icon[name]:SetPoint(db.iconPos, SBC_block[name])
							SBC_display[name]:ClearAllPoints()
							SBC_display[name]:SetPoint(db.iconPos, SBC_icon[name],
								db.iconPos == "LEFT" and "RIGHT" or "LEFT"
							)
							SBC_display[name]:SetWidth(db.plugins[name].width - 20)
						end
					end,
					disabled = function() return not obj.icon or not pDB.enabled end,
				},
			},
		}
	end
	return blockOpts[name]
end

------------------------------
--      Main Functions      --
------------------------------

SBC:RegisterEvent("ADDON_LOADED")
SBC:RegisterEvent("PLAYER_LOGIN")
SBC:SetScript("OnEvent", function(_, event, ...) SBC[event](SBC, ...) end)

function SBC:TextUpdate(_, name, _, data)
	SBC_display[name]:SetText(data)
end

local txtfmt = "%s|cff%s%s|r"
function SBC:ValueUpdate(_, name, _, data, obj)
	SBC_display[name]:SetFormattedText(txtfmt, data, db.plugins[name].labelc, obj.suffix)
end

function SBC:LauncherUpdate(name, data)
	SBC_display[name]:SetText(data)
end

function SBC:Tooltip(evt, name, _, func, obj)
	if evt:find("OnTooltipShow") then
		SBC_block[name]:SetScript("OnEnter", function()
			SBC_block[name]:SetAlpha(db.plugins[name].shownAlpha)
			if not this.isMoving and func then
				GameTooltip:SetOwner(SBC_block[name], "ANCHOR_CURSOR")
				func(SBC_block[name])
				GameTooltip:Show()
			end
		end)
		SBC_block[name]:SetScript("OnLeave", function() GameTooltip:Hide()
			SBC_block[name]:SetAlpha(db.plugins[name].hiddenAlpha)
		end)
	else
		SBC_block[name]:SetScript("OnEnter", function()
			SBC_block[name]:SetAlpha(db.plugins[name].shownAlpha)
			if not this.isMoving and func then
				func(SBC_block[name])
			end
		end)
		SBC_block[name]:SetScript("OnLeave", function()
			if obj.OnLeave then obj.OnLeave(SBC_block[name]) end
			SBC_block[name]:SetAlpha(db.plugins[name].hiddenAlpha)
		end)
	end
end

function SBC:Click(_, name, _, func)
	SBC_block[name]:SetScript("OnMouseUp", function(_, btn) if not this.isMoving then func(SBC_block[name], btn) end end)
end

function SBC:Icon(_, name, _, icon)
	SBC_icon[name]:SetTexture(icon)
end

local bg = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",}
function SBC:New(_, name, obj)
	_G.LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("StatBlock_"..name, getBlockOptions)
	_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StatBlock_"..name, name, "StatBlockCore")

	if not db.plugins[name].enabled then return end

	if obj.suffix then
		obj.suffix = db.plugins[name].label or " ".. obj.suffix
	elseif obj.launcher then
		obj.label = db.plugins[name].label or obj.label or obj.tocname or name
	end

	SBC_block[name] = _G.CreateFrame("Frame", "StatBlockCore_"..name, UIParent)
	SBC_block[name]:SetBackdrop(bg)
	SBC_block[name]:SetBackdropColor(db.globalBDC.r,db.globalBDC.g,db.globalBDC.b,db.globalAlpha)
	SBC_block[name]:EnableMouse(true)
	if not db.plugins[name].lock then SBC_block[name]:SetMovable(true) SBC_block[name]:EnableMouseWheel(true) end
	SBC_block[name]:SetFrameStrata(db.plugins[name].strata)
	SBC_block[name]:RegisterForDrag("LeftButton")
	SBC_block[name]:SetScript("OnDragStart", function() if this:IsMovable() then this.isMoving = true this:StartMoving() GameTooltip:Hide() end end)
	SBC_block[name]:SetScript("OnMouseWheel", function(_, d)
		if db.plugins[name].lock then return end
		local w = SBC_block[name]:GetWidth() + d
		SBC_block[name]:SetWidth(w) db.plugins[name].width = w
		SBC_display[name]:SetWidth(obj.icon and db.plugins[name].icon and w-20 or w)
	end)
	SBC_block[name]:SetScript("OnDragStop", function() this:StopMovingOrSizing()
		--try to stick to a screen edge, then try to stick to a bar [ Thank you Tuller <3 ]
		for _,v in pairs(SBC_block) do
			if FlyPaper.Stick(this, v, 20, 2, 2) then
				break
			end
		end
		for k,v in pairs(SBC_block) do
			local s = v:GetEffectiveScale()
			db.plugins[k].posx = v:GetLeft() * s db.plugins[k].posy = v:GetTop() * s
		end
		this.isMoving = nil
	end)
	SBC_block[name]:SetWidth(db.plugins[name].width)
	SBC_block[name]:SetHeight(20)
	SBC_block[name]:Show()
	SBC_block[name]:SetScale(db.plugins[name].scale)
	SBC_block[name]:SetAlpha(db.plugins[name].hiddenAlpha)

	if db.plugins[name].posx == 0 and db.plugins[name].posy == 0 then
		SBC_block[name]:ClearAllPoints()
		SBC_block[name]:SetPoint("CENTER", UIParent, "CENTER", math.random(-150,150), math.random(-150,150))
	else
		local s = SBC_block[name]:GetEffectiveScale()
		SBC_block[name]:ClearAllPoints()
		SBC_block[name]:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.plugins[name].posx / s, db.plugins[name].posy / s)
	end
	SBC_block[name]:SetParent(db.plugins[name].parent)

	SBC_display[name] = SBC_block[name]:CreateFontString("StatBlockCoreFont_"..name, "OVERLAY")
	if obj.icon and db.plugins[name].icon then
		SBC_icon[name] = SBC_block[name]:CreateTexture("StatBlockCoreIcon_"..name,"BACKGROUND")
		SBC_icon[name]:SetTexture(obj.icon)
		SBC_icon[name]:SetWidth(20)
		SBC_icon[name]:SetHeight(20)
		SBC_icon[name]:SetPoint(db.iconPos, SBC_block[name])
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_icon", "Icon")
		SBC_display[name]:SetPoint(db.iconPos, SBC_icon[name], db.iconPos == "LEFT" and "RIGHT" or "LEFT")
		SBC_display[name]:SetWidth(db.plugins[name].width - 20)
	else
		SBC_display[name]:SetPoint("CENTER", SBC_block[name], "CENTER")
		SBC_display[name]:SetWidth(db.plugins[name].width)
	end
	SBC_display[name]:SetFont(media:Fetch("font", db.globalFont), db.globalSize, db.outline)
	SBC_display[name]:SetTextColor(db.plugins[name].color.r, db.plugins[name].color.g, db.plugins[name].color.b)
	SBC_display[name]:SetJustifyH(db.plugins[name].align)
	if db.shadow then
		SBC_display[name]:SetShadowColor(0, 0, 0, 1)
		SBC_display[name]:SetShadowOffset(1, -1)
	end

	if obj.suffix then
		self:ValueUpdate(nil, name, nil, obj.value or name, obj)
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_value", "ValueUpdate")
	elseif obj.launcher then
		self:LauncherUpdate(name, obj.label)
	else
		self:TextUpdate(nil, name, nil, obj.text or name, obj)
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_text", "TextUpdate")
	end
	if obj.OnEnter then
		self:Tooltip("OnEnter", name, nil, obj.OnEnter, obj)
	elseif obj.OnTooltipShow then
		self:Tooltip("OnTooltipShow", name, nil, obj.OnTooltipShow, obj)
	else
		self:Tooltip("OnEnter", name, nil, nil, obj)
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_OnEnter", "Tooltip")
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_OnTooltipShow", "Tooltip")
	end
	if obj.OnClick then
		self:Click(nil, name, nil, obj.OnClick)
	else
		ldb.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name.."_OnClick", "Click")
	end
end

function SBC:ADDON_LOADED(msg)
	if msg == "StatBlockCore" then
		local defaults = {
			profile = {
				globalFont = "Friz Quadrata TT",
				globalSize = 20,
				globalColor = { r = 1, g = 1, b = 1 },
				globalBDC = { r = 1, g = 1, b = 1 },
				globalAlpha = 1,
				globalLabelColor = { r = 1, g = 1, b = 1 },
				iconPos = "LEFT",
				shadow = true,
				outline = "NONE",
				plugins = {
					["*"] = {
						color = { r = 1, g = 1, b = 1, },
						labelColor = { r = 1, g = 1, b = 1 },
						labelc = "ffffff",
						scale = 1,
						align = "CENTER",
						width = 100,
						posx = 0,
						posy = 0,
						parent = "UIParent",
						strata = "BACKGROUND",
						icon = true,
						enabled = true,
						shownAlpha = 1,
						hiddenAlpha = 1,
					},
				},
			},
		}
		self:UnregisterEvent("ADDON_LOADED")
		self.db = _G.LibStub("AceDB-3.0"):New("StatBlockCoreDB", defaults)
		db = self.db.profile
		ldb.RegisterCallback(self, "LibDataBroker_DataObjectCreated", "New")

		local txt = "|cff33ff99StatBlocks|r: "
		_G.LibStub("AceConfig-3.0"):RegisterOptionsTable("StatBlockCore", getOptions)
		_G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StatBlockCore")
		_G.SlashCmdList["STATBLOCKCORE_MAIN"] = function()
			InterfaceOptionsFrame_OpenToFrame("StatBlockCore")
			_G.ChatFrame1:AddMessage(txt.. L["Open_Alert"])
		end
		_G.SLASH_STATBLOCKCORE_MAIN1 = "/sbc"
		_G.SLASH_STATBLOCKCORE_MAIN2 = "/statblockcore"

		local function profilePrint() _G.ChatFrame1:AddMessage(txt.. L["Reload UI"]) end
		self.db.RegisterCallback(self, "OnProfileReset", profilePrint)
		self.db.RegisterCallback(self, "OnProfileChanged", profilePrint)
		SBC.ADDON_LOADED = nil
	end
end

function SBC:PLAYER_LOGIN()
	for k,v in pairs (SBC_block) do
		if db.plugins[k].posx == 0 and db.plugins[k].posy == 0 then
			v:ClearAllPoints()
			v:SetPoint("CENTER", UIParent, "CENTER", math.random(-150,150), math.random(-150,150))
		else
			local s = v:GetEffectiveScale()
			v:ClearAllPoints()
			v:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.plugins[k].posx / s, db.plugins[k].posy / s)
		end
	end
	SBC:SetScript("OnEvent", nil)
	for name, obj in ldb:DataObjectIterator() do
		if not SBC_block[name] and db.plugins[name].enabled then
			self:New(nil, name, obj)
		end
	end
	SBC.PLAYER_LOGIN = nil
end

------------------------------
--         Cleanup          --
------------------------------

local prev = 0
SBC:SetScript("OnUpdate", function(_, c)
	prev = prev + c
	if prev > 7 then
		SBC:SetScript("OnUpdate", nil)
		ldb.UnregisterCallback(SBC, "LibDataBroker_DataObjectCreated")
		SBC.New = nil
		SBC.Click = nil
		SBC.Tooltip = nil
		prev = nil
		for k in pairs(SBC_block) do
			ldb.UnregisterCallback(SBC, "LibDataBroker_AttributeChanged_"..k.."_OnClick")
			ldb.UnregisterCallback(SBC, "LibDataBroker_AttributeChanged_"..k.."_OnEnter")
			ldb.UnregisterCallback(SBC, "LibDataBroker_AttributeChanged_"..k.."_OnTooltipShow")
		end
	end
end)

